/*==========================================================================
    CGL equation
      $B"_(Bt w =c0*w-c2*norm(w)*w+c1*$B"$(Bw

      v = spatial average of w(x,y);
      usually, c0=1,real(c1)=real(c2)=1.0;
===========================================================================*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <math.h>
#include <XRS/Complex_Control.h>
#include <XRS/Graphic.h>
#include <XRS/Plane.h>

#define  Random   drand48()
#define  Color    6


/************** $BBg0hJQ?t$H4X?t@k8@(B ********************/

double    c02=0,c1=2,c2=-2; // $B%Q%i%a!<%?(B
Complex   *w;                             // $B>l(B
double    L;                              // $B%7%9%F%`%5%$%:(B
double    t;                              // $B;~4V(B

void            initial(int);


/*---------------- $B0J2<(B XWindow $BMQ(B --------------------*/

Complex_Control_Window CW;
Graphic_Window  W,SW;
Plane_Window    PW;
int             Wimax,Wjmax,PWsize,Dispstep=50;
int             Initial=0;
double          Amp=0.8;

unsigned long*  colordisk();
unsigned long*  greyscaledisk();
int             diskcolor(Complex w);
void            draw();
void            SW_draw();
void            change_color(int);
void            explain(int)
{  if(!fork()){ execlp("xdvi","xdvi","-s","8","README_CGL_1d.dvi",0);}  }



int main(int argc,char **argv)
{
   int       i,ts;   
   double    DT,DX;
   Complex  *w2,t0,t1,t2;

   /*--------------- initialize ---------------------*/
   
   if(argc>1) L=atof(argv[1]);  else L=200;
   if(argc>2) DX=atof(argv[2]); else DX=1;
   if(argc>3) DT=atof(argv[3]); else DT=0.02;
   if(argc>4) c1=atof(argv[4]); else c1=1;
   if(argc>5) c2=atof(argv[5]); else c2=-1;
   if(argc>6) c02=atof(argv[6]); else c02=0;   

   fprintf(stderr,"usage%% CGL_1d [L] [DX] [DT] [c1] [c2] [c0-c2]\n"
          " current systemsize: L=%g\n"
          " current parameter : c1=%g c2=%g c0-c2=%g\n"
	  " space and time meshsize: DX=%g DT=%g\n"
   	  ,L,c1,c2,c02,DX,DT); 

   Wimax=(int)(L/DX); Wjmax=200; PWsize=200;
   w  = new Complex [Wimax];   
   w2 = new Complex [Wimax];   

   /*---------------- setup window -------------------*/
   
   CW.realtime=0;
       f_dialog d(CW,explain);
       d.resize(2*d.width(),d.height());
       d.string(20);
       d.title(" readme (Click here)");
   CW.realtime=1;
   
   CW<<t<<"t"<<initial<<"initial"<<PARAMETER
     << c1 << c2 << c02   << Amp << Dispstep
     <<"c1"<<"c2"<<"c0-c2"<<"Amp"<<"slow";
   
   W.geometry(Wimax,Wjmax,1)<<"CGL space-time series"
        <<COLOR(colordisk(),(2*Color+1)*(2*Color+1));
   
   SW.geometry(2*Color+1,2*Color+1,4)<<"color sample"
       <<COLOR(colordisk(),(2*Color+1)*(2*Color+1))<<change_color;

   PW.geometry(PWsize,PWsize)<<"complex plane";
   
   initialize();
   SW_draw();
   
   /*--------------- main routine -------------------*/

   initial(0);
   while(1){
      checkevent();

      t0=Complex(1.0,c02+c2)*DT;
      t1=Complex(1.0,c1)*DT/(DX*DX);
      t2=Complex(1.0,c2)*DT;

      for(ts=0; ts<Dispstep;ts++){

         for(i=1; i<Wimax-1; i++)
            w2[i]=w[i]+t1*(w[i-1]+w[i+1]-2.0*w[i]);
	  
         for(i=1; i<Wimax-1; i++)
            w[i]=w2[i]+(t0-t2*norm(w[i]))*w[i];

	 w[0]=w[Wimax-2];
	 w[Wimax-1]=w[1];
      }
      
      t+=DT*Dispstep;
      draw();
   }
}


/**********************  $B=i4|>r7o(B *************************/

void initial(int)
{  
   for(int i=0; i<Wimax; i++)
      w[i]=polar((double)Random,(double)(2*M_PI*Random));

   w[0]=w[Wimax-2];
   w[Wimax-1]=w[1];
   t=0; Initial=1;
}


/********************** $B=q$/(B *************************/

#define PWinI(x) ((int)(PWsize*0.5*(Amp*(x)+1.0)))
#define PWinJ(x) ((int)(PWsize*0.5*(-Amp*(x)+1.0)))		  
		  
void draw(void)
{
    /*======= space-time series =========*/
    
    static int j;    
    if(Initial){ W.clear(); j=Wjmax-1; }
    
    for(int i=0; i<Wimax; i++)
       W.point(diskcolor(Amp*w[i]),i,j);

    j--;
    if(j<0) j=Wjmax-1;

    
    /*======= complex plane snapshot =========*/

    if(Initial){
	PW.clear();
	PW.draw_line(1,PWinI(0),0,PWinI(0),PWsize);
	PW.draw_line(1,0,PWinJ(0),PWsize,PWinJ(0));	
    }
    
    static int i0,i1,j0,j1;
    PW.clear(0);
    i0=PWinI(real(w[0])); j0=PWinJ(imag(w[0]));	     
    for(int i=1; i<Wimax; i++){
       i1=PWinI(real(w[i])); j1=PWinJ(imag(w[i]));	     
       PW.draw_line(0,i0,j0,i1,j1);
       i0=i1; j0=j1;	     
    }

    if(Initial) Initial=0; 
}


void change_color(int)
{
    static int color=1;

    if(color){
	W <<COLOR(greyscaledisk(),(2*Color+1)*(2*Color+1));
	SW<<COLOR(greyscaledisk(),(2*Color+1)*(2*Color+1));
	color=0;
    }else{
	W <<COLOR(colordisk(),(2*Color+1)*(2*Color+1));
        SW<<COLOR(colordisk(),(2*Color+1)*(2*Color+1));
	color=1;
    }
    
    SW_draw();
}    


void SW_draw()
{
    for(int i=0; i<2*Color+1; i++)
       for(int j=0; j<2*Color+1; j++)
  	   SW.point(i+j*(2*Color+1),i,2*Color-j);
}


/********************** $B?'$NG[Ns(B ***********************/

unsigned long* colordisk()
{
    unsigned long *cp=new unsigned long [(2*Color+1)*(2*Color+1)];
    double r,phi;
    int i,j,k;
    for(i=-Color; i<Color+1; i++)
        for(j=-Color; j<Color+1; j++){
            k=i+Color+(j+Color)*(2*Color+1);
            r=sqrt((double)(i*i+j*j))/(double)Color;
            if(i==0 && j==0) phi=0; 
            else             phi=atan2((double)i,(double)j)/(2*M_PI);
            cp[k]=color_pixel(phi,r,1);
        }
    return cp;
}    


unsigned long* greyscaledisk()
{
    unsigned long *cp=new unsigned long [(2*Color+1)*(2*Color+1)];
    double r;
    int i,j,k;
    for(i=-Color; i<Color+1; i++)
        for(j=-Color; j<Color+1; j++){
            k=i+Color+(j+Color)*(2*Color+1);
            r=sqrt((double)(i*i+j*j))/(double)Color;  if(r>1) r=1; 
            cp[k]=color_pixel(0,0,1-r);	    
        }
    return cp;
}    


/*------------------- $BG[Ns$NHV9f$rJV$9(B ----------------*/

int diskcolor(Complex w)
{
    static int cn=2*Color+1;
    w+=Complex(1,1); w*=Color;
    return (int)real(w)+(int)imag(w)*cn;
}




