/*==========================================================================
    global coupled complex Ginzburg Landau equation
      dwdt =c0*w-c2*norm(w)*w+K*c1*(<w>-w)

      <w> = average;
      usually, c0=1,real(c1)=real(c2)=1.0;
===========================================================================*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <math.h>
#include <XRS/Complex_Control.h>
#include <XRS/Graphic.h>
#include <XRS/Plane.h>

#define  Random   drand48()
#define  Color    6


/************** $BBg0hJQ?t$H4X?t@k8@(B ********************/

double    c1,c2,c02,K;                    // $B%Q%i%a!<%?(B
Complex   *w;                             // $B>l(B
int       N;                              // $B%7%9%F%`%5%$%:(B
double    t;                              // $B;~4V(B

void            initial(int);


/*---------------- $B0J2<(B XWindow $BMQ(B --------------------*/

Complex_Control_Window CW;
Plane_Window    PW;
int             PWsize=200;
int             Initial=0,Trace=1;
double          Amp=0.8;

void            draw();
void            trace(int){ Trace=!Trace; }
void            explain(int)
{  if(!fork()){ execlp("xdvi","xdvi","-s","8","README_GCGL.dvi",0);}  }


int main(int argc,char **argv)
{
   int       i;   
   double    DT;
   Complex   W,t0,t1,t2;

   /*--------------- initialize ---------------------*/
   
   if(argc>1) N =atoi(argv[1]);  else N=500;
   if(argc>2) DT=atof(argv[2]);  else DT=0.02;
   if(argc>3) c1=atof(argv[3]);  else c1=2;
   if(argc>4) c2=atof(argv[4]);  else c2=-2;
   if(argc>5) K =atof(argv[5]);  else K=0.5;   
   if(argc>6) c02=atof(argv[6]); else c02=0.4;   

   fprintf(stderr,"usage%% GCGL [N] [DT] [c1] [c2] [K] [c02]\n"
          " current systemsize: N=%d\n"
          " current parameter : c1=%g c2=%g K=%g c0-c2=%g\n"
	  " space and time meshsize: DT=%g\n"
   	  ,N,c1,c2,K,c02,DT); 

   PWsize=200;
   w  = new Complex [N];   

   /*---------------- setup window -------------------*/
   
   CW.realtime=0;
       f_dialog d(CW,explain);
       d.resize(2*d.width(),d.height());
       d.string(20);
       d.title(" readme (Click here)");
   CW.realtime=1;
   
   CW<<t<<"t"<<initial<<"initial"<<PARAMETER
     << c1 << c2 << K << c02 << Amp 
     <<"c1"<<"c2"<<"K"<<"c02"<<"Amp";
   
   PW.geometry(PWsize,PWsize)<<"complex plane"<<trace;
   
   initialize();
   
   /*--------------- main routine -------------------*/

   initial(0);
   while(1){
      checkevent();

      t0=Complex(1.0,c02+c2)*DT;
      t1=Complex(1.0,c2)*DT;
      t2=Complex(1.0,c1)*K*DT;

      W=0; for(i=0; i<N; i++) W+=w[i];  W/=N;
      for(i=0; i<N; i++) w[i]+=(t0-t1*norm(w[i]))*w[i]+t2*(W-w[i]);

      t+=DT;
      draw();
   }
}


/**********************  $B=i4|>r7o(B *************************/

void initial(int)
{  
   for(int i=0; i<N; i++)
      w[i]=polar((double)Random,(double)(2*M_PI*Random));

   t=0; Initial=1;
}


/********************** $B=q$/(B *************************/

#define PWinI(x) ((int)(PWsize*0.5*(Amp*(x)+1.0)))
#define PWinJ(x) ((int)(PWsize*0.5*(-Amp*(x)+1.0)))		  
		  
void draw(void)
{
    /*======= complex plane snapshot =========*/

    if(Initial){
	PW.clear();
	PW.draw_line(1,PWinI(0),0,PWinI(0),PWsize);
	PW.draw_line(1,0,PWinJ(0),PWsize,PWinJ(0));	
    }
    
    if(Trace) PW.clear(0);
    for(int i=0; i<N; i++){
       PW.draw_point(0,PWinI(real(w[i])),PWinJ(imag(w[i])));
    }

    PW.clear(2);
    PW.draw_circle(2,PWinI(real(w[0])),PWinJ(imag(w[0])),4);

    if(Initial) Initial=0; 
}


