/*=======================================================================

     Kuramoto Shivashinsky $BJ}Dx<0(B

     $B"_(Bt U = - $B"_(Bxx U + ($B"_(BxU)($B"_(BxU) - $B"_(Bxxxx U
     V=2$B"_(BxU $B$GJQ49(B
     $B"_(Bt V = - $B"_(Bxx V - $B"_(Bxxxx V + V$B"_(BxV

========================================================================*/  

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <math.h>
#include <XRS/Control.h>
#include <XRS/Plane.h>
#include <XRS/Simple.h>

#define PI          M_PI
#define integer(x)  ((int)(floor(x)+floor(2*((x)-floor(x)))))
#define RAN         (drand48()-0.5)
#define XMark(d,w,gc,x,y,r) XDrawArc(d,w,gc,x-r,y-r,2*r,2*r,0,23040);


/************** $BBg0hJQ?t$H4X?t@k8@(B ********************/

double              t,L;
double              *v;
double              DT,DX,v_start;
int                 wn;

void initial(int);
void draw(int);
void noise(int);

/*---------------- $B0J2<(B XWindow $BMQ(B --------------------*/

Plane_Window    W;
Simple_Window   SW;
Control_Window  CW;
Display         *d;
GC              black,white;
double          dispv=0.1;
int             Wimax,Wjmax,Initial=0,DISPSTEP=50,SWimax,SWjmax;

void            explain(int)
{  if(!fork()){ execlp("xdvi","xdvi","-s","8","README_KS_1d.dvi",0);}  }


/********** main ***********/

int main(int argc,char **argv)
{
    if(argc>1) L=atof(argv[1]);      else L=200;
    if(argc>2) DX=atof(argv[2]);     else DX=0.5;
    if(argc>3) DT=atof(argv[3]);     else DT=0.005;
    if(argc>4) wn=atoi(argv[4]);     else wn=10;

    fprintf(stderr,"usage%% KS [L] [DX] [DT] [wn]\n"
          " current systemsize: L=%g\n"
	  " space and time meshsize: DX=%g DT=%g\n"
	  " initial wave number=2$B&P(Bwn: wn=%d\n"
   	  ,L,DX,DT,wn); 

    Wimax=integer(L/DX)+3;  Wjmax=200;
    SWimax=Wimax;           SWjmax=500;
    v_start=1;

    double   *v2;
    v =new double [Wimax+1];     v2=new double [Wimax+1];    
    
/*----- initialize ----------*/

    CW.realtime=0;
       f_dialog readme(CW,explain);
       readme.resize(2*readme.width(),readme.height());
       readme.string(20);
       readme.title(" readme (Click here)");
    CW.realtime=1;
    
    CW<< initial << noise << t 
      <<"initial"<<"noise"<<"t"
      <<PARAMETER<< wn <<DISPSTEP<<dispv
                 <<"wn"<< "slow" << "v*";

    W.geometry(Wimax,Wjmax)<<"KS(snapshot)";
    SW.geometry(SWimax,SWjmax)<<"space-time series";

    initialize();  
    d=W.disp(); 
    white=graphic_context("white");
    black=graphic_context("black");

   /*--------------- main roution -------------------*/    
    initial(0);
    draw(1);
    
    int      j,ts;
    double   dv,dv2,dv4,DTxx,DTx4;

    DTxx=DT/(DX*DX);
    DTx4=DT/(DX*DX*DX*DX);    
     
/*----- main routine --------*/

    while(1){
         checkevent();     

         for(ts=0; ts<DISPSTEP; ts++){
            for(j=2;j<Wimax-1;j++){
               dv =0.5*(v[j+1]-v[j-1]);
	       dv2=v[j+1]+v[j-1]-2.0*v[j];
	       dv4=v[j+2]+v[j-2]-4.0*(v[j+1]+v[j-1])+6.0*v[j];
	       v2[j]=v[j]+DTxx*(-dv2+v[j]*dv)-DTx4*dv4;
            }

	    //	    U=0; for(j=2; j<Wimax-1; j++) U+=v2[j];  U/=(double)(Wimax-3);

            for(j=2;j<Wimax-1;j++){
	      v[j]=v2[j];//-U;
	    }
	    
            v[0]=v[Wimax-3]; v[Wimax]=v[3];
            v[1]=v[Wimax-2]; v[Wimax-1]=v[2];

            t += DT;
        }

        draw(Initial);
        if(Initial) Initial=0;
    }
     
    delete [] v;    delete [] v2;
}


/*************** initialize function ************/

void initial(int)
{
     for(int j=2; j<Wimax-1; j++) v[j]=0;     
     noise(0); 
     Initial=1;  t=0;
}


void noise(int)
{
    double phi=2*PI*RAN;
    double LL=(double)(Wimax-3);
    double a=2.0*PI*wn/LL;
    
    for(int j=2; j<Wimax-1; j++)
      v[j]+=v_start*cos(a*(j-2)-phi);
	    
    v[0]=v[Wimax-3]; v[Wimax]=v[3];
    v[1]=v[Wimax-2]; v[Wimax-1]=v[2];
}


void draw(int initial)
{
    int m,j;
    static int *pv=new int [Wimax];
    static int  sj,jp; 
    double Ampv=0.4*dispv*(double)Wjmax;

    if(initial){ W.clear(); 
                 W.draw_line(3,0,(int)(0.5*Wjmax),Wimax,(int)(0.5*Wjmax)); }

    sj--; 
    if(initial || sj<50){ SW.clear(); sj=SWjmax-50;}

    for(m=0; m<Wimax; m++){
       j=-integer(Ampv*v[m]); 
       if(sj%5==0 && m>0) XDrawLine(d,SW,black,m-1,jp+sj,m,j+sj);
       jp=j;
       j=j*2+Wjmax/2;
       if(pv[m]!=j){
          W.erase_point(0,m,pv[m]); 
          W.draw_point(0,m,j);
          pv[m]=j;
       }
    }
}








