/*==========================================================================
    Newell-Whitehead equation
      $B"_(Bt w=$B&L(Bw-norm(w)*w+($B"_(Bx-i/2kc$B"_(By)^2 w
===========================================================================*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <unistd.h>
#include <sys/wait.h>

#include <XRS/Complex_Control.h>
#include <XRS/Graphic.h>

#define  Random   drand48()
#define  Dispstep 10
#define  Color    6

/************** $BBg0hJQ?t$H4X?t@k8@(B ********************/

double    mu,kc;  // $B%Q%i%a!<%?(B
Complex   **w;    // $B>l(B
double    LX,LY;  // $B%7%9%F%`%5%$%:(B
double    DX,DY;  // $B6u4V%a%C%7%e%5%$%:(B
double    t;      // $B;~4V(B
int       nw=-10; // $B=i4|>r7o$NGH?t(B

void            randinitial(int);
void            expinitial(int);
void            boundary();


/*---------------- $B0J2<(B XWindow $BMQ(B --------------------*/

Complex_Control_Window CW;
Graphic_Window  W,SW;
int             Wimax,Wjmax;
int             Initial=0;
double          Amp=0.8;

unsigned long*  colordisk();
int             diskcolor(Complex w);
void            draw();
void            SW_draw();
void            explain(int)
{  if(!fork()){ execlp("xdvi","xdvi","-s","8","README_NW.dvi",0);}  }


int main(int argc,char **argv)
{
   int       i,j,ts;   
   double    DT;
   Complex   **w2,dwxx,dwxy,dwyy,t0,t1,t2,t3;

   /*--------------- initialize ---------------------*/
   
   if(argc>1) LX=atof(argv[1]); else LX=100;
   if(argc>2) LY=atof(argv[2]); else LY=LX;
   if(argc>3) DX=atof(argv[3]); else DX=1.0;  DY=DX;
   if(argc>4) DT=atof(argv[4]); else DT=0.1;
   if(argc>5) mu=atof(argv[5]); else mu=1.0; 
   if(argc>6) kc=atof(argv[6]); else kc=1.0;
   
   fprintf(stderr,"usage%% NW [LX] [LY] [DX] [DT] [mu] [kc]\n"
          " current systemsize: (LX,LY)=(%g,%g)\n"
          " current parameter : mu=%g kc=%g\n"
	  " space and time meshsize: DX=%g DT=%g\n"
   	  ,LX,LY,mu,kc,DX,DT); 
   
   Wimax=(int)(LX/DX)+2;
   Wjmax=(int)(LY/DY)+4;
   array(w,Wimax,Wjmax);   
   array(w2,Wimax,Wjmax);   


   /*---------------- setup window -------------------*/

   CW.realtime=0;
       f_dialog d(CW,explain);
       d.resize(2*d.width(),d.height());
       d.string(20);
       d.title(" readme (Click here)");
   CW.realtime=1;
   
   CW<<t<<"t"<<randinitial<<"random"<<expinitial<<"stationary"
       <<PARAMETER
       << nw << mu << kc << Amp
       <<"nw"<<"mu"<<"kc"<<"Amp";
   
   W.geometry(Wimax,Wjmax,2)<<"Newell Whitehead equation"
       <<COLOR(colordisk(),(2*Color+1)*(2*Color+1));

   SW.geometry(2*Color+1,2*Color+1,4)<<"color sample"
       <<COLOR(colordisk(),(2*Color+1)*(2*Color+1));

   initialize();
   randinitial(0);

   SW_draw();
   draw();
   
   /*--------------- main roution -------------------*/

   while(1){
      checkevent();

      t0=mu*DT;
      t1=DT/(DX*DX);
      t2=-DT/(kc*DX*DY*DY)*Complex(0.0,1.0);
      t3=-DT/(4.0*kc*kc*DY*DY*DY*DY);

      for(ts=0; ts<Dispstep;ts++){

         for(i=1; i<Wimax-1; i++)
	    for(j=2; j<Wjmax-2; j++){
	       dwxx=w[i-1][j]+w[i+1][j]-2.0*w[i][j];
	       dwxy=0.5*(w[i+1][j+1]-w[i-1][j+1]
			 +w[i+1][j-1]-w[i-1][j-1])-w[i+1][j]+w[i-1][j];
	       dwyy=w[i][j+2]+w[i][j-2]-4.0*(w[i][j+1]+w[i][j-1])+6.0*w[i][j];
               w2[i][j]=w[i][j]+t1*dwxx+t2*dwxy+t3*dwyy;
	    }
	  
         for(i=1; i<Wimax-1; i++)
	    for(j=2; j<Wjmax-2; j++)
               w[i][j]=w2[i][j]
		      +(t0-DT*norm(w[i][j]))*w[i][j];

	 boundary();
      }
      
      t+=DT*Dispstep;
      draw();
   }
}


/**********************  $B=i4|>r7o(B *************************/

void randinitial(int)
{  
   for(int i=0; i<Wimax; i++)
      for(int j=0; j<Wjmax; j++)
          w[i][j]=polar((double)Random,(double)(2*M_PI*Random));

   boundary();
   t=0; Initial=1;
}


void expinitial(int)
{
   double x,q;
   for(int i=0; i<Wimax; i++)
      for(int j=0; j<Wjmax; j++){
	  x=nw*(double)(i-1)/(Wimax-2)+0.0000001*Random;
	  q=2*M_PI*nw/(DX*(Wimax-2));
	  w[i][j]=sqrt(mu-q*q)*exp(Complex(0,2*M_PI*x));
      }	  

   boundary();
   t=0; Initial=1;
}


/**********************  $B6-3&>r7o(B *************************/

void boundary()
{
// periodic boundary condition
    for(int j=0;j<Wjmax;j++){
        w[0][j]=w[Wimax-2][j];
        w[Wimax-1][j]=w[1][j];
    }
    
    for(int i=0;i<Wimax;i++){
        w[i][0]=w[i][Wjmax-4];
        w[i][1]=w[i][Wjmax-3];
        w[i][Wjmax-2]=w[i][2];
        w[i][Wjmax-1]=w[i][3];
    }
}    


/********************** $B=q$/(B *************************/

void draw(void)
{
    if(Initial){ W.clear(); Initial=0; }

    for(int i=0; i<Wimax; i++)
       for(int j=0; j<Wjmax; j++)
  	   W.point(diskcolor(Amp*w[i][j]),i,j);
}


void SW_draw()
{
    for(int i=0; i<2*Color+1; i++)
       for(int j=0; j<2*Color+1; j++)
  	   SW.point(i+j*(2*Color+1),i,2*Color-j);
}


/********************** $B?'$NG[Ns(B ***********************/

unsigned long* colordisk()
{
    unsigned long *cp=new unsigned long [(2*Color+1)*(2*Color+1)];
    double r,phi;
    int i,j,k;
    for(i=-Color; i<Color+1; i++)
        for(j=-Color; j<Color+1; j++){
            k=i+Color+(j+Color)*(2*Color+1);
            r=sqrt((double)(i*i+j*j))/(double)Color;
            if(i==0 && j==0) phi=0; 
            else             phi=atan2((double)i,(double)j)/(2*M_PI);
            cp[k]=color_pixel(phi,r,1);
        }
    return cp;
}    


/*------------------- $BG[Ns$NHV9f$rJV$9(B ----------------*/

int diskcolor(Complex w)
{
    static int cn=2*Color+1;
    w+=Complex(1,1); w*=Color;
    return (int)real(w)+(int)imag(w)*cn;
}

