/*==========================================================================
    bistable system
      $B&E"_(Bt u =u-u^3-v+$B&E"_(Bxx u
===========================================================================*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <math.h>
#include <XRS/Control.h>
#include <XRS/Plane.h>

#define  Random   drand48()

/************** $BBg0hJQ?t$H4X?t@k8@(B ********************/

double    ep,v;                           // $B%Q%i%a!<%?(B
double   *u;                              // $B>l(B
double    L;                              // $B%7%9%F%`%5%$%:(B
double    t;                              // $B;~4V(B

void            initial(int);


/*---------------- $B0J2<(B XWindow $BMQ(B --------------------*/

Control_Window  CW;
Plane_Window    W,PW;
int             Wimax,Wjmax,PWsize=200,Dispstep=500,Clear=1;
int             Initial=0;

void            draw();
void            PW_draw();
void            explain(int)
{  if(!fork()){ execlp("xdvi","xdvi","-s","8","README_bistable.dvi",0);}  }
void            show(int){ Clear=!Clear; }


int main(int argc,char **argv)
{
   int       i,ts;   
   double    DT,DX;
   double   *u2,t0,t1;

   /*--------------- initialize ---------------------*/
   
   if(argc>1) L=atof(argv[1]);  else L=200;
   if(argc>2) DX=atof(argv[2]); else DX=0.2;
   if(argc>3) DT=atof(argv[3]); else DT=0.0005;
   if(argc>4) ep=atof(argv[4]); else ep=0.1;
   if(argc>5) v=atof(argv[5]);  else v=-0.2;

   fprintf(stderr,"usage%% bistable [L] [DX] [DT] [ep] [v]\n"
          " current systemsize: L=%g\n"
          " current parameter : ep=%g v=%g\n"
	  " space and time meshsize: DX=%g DT=%g\n"
   	  ,L,ep,v,DX,DT); 

   Wimax=(int)(L/DX); Wjmax=200; 
   u  = new double [Wimax];   
   u2 = new double [Wimax];   

   /*---------------- setup window -------------------*/
   
   CW.realtime=0;
       f_dialog d(CW,explain);
       d.resize(2*d.width(),d.height());
       d.string(20);
       d.title(" readme (Click here)");
   
       f_dialog d2(CW,show);
       d2.resize(2*d2.width(),d2.height());
       d2.string(20);
       d2.title(" the way to showing ");
   CW.realtime=1;
   
   CW<<t<<"t"<<PARAMETER
     << ep << v <<Dispstep
     <<"ep"<<"v"<<"slow";
   
   W.geometry(Wimax,Wjmax,1)<<initial
       <<"bistable system( A mouse click sets up the initial condition. )";
	   
   PW.geometry(PWsize,PWsize,1)<<"u-u^3-v";	   
   
   initialize();
   
   /*--------------- main roution -------------------*/

   initial(0);
   while(1){
      checkevent();
      if(CW.chev) PW_draw();

      t0=DT/ep;
      t1=t0/(DX*DX);

      for(ts=0; ts<Dispstep;ts++){

         for(i=1; i<Wimax-1; i++)
            u2[i]=u[i]+t1*(u[i-1]+u[i+1]-2.0*u[i]);
	  
         for(i=1; i<Wimax-1; i++)
            u[i]=u2[i]+t0*((1.0-u[i]*u[i])*u[i]-v);

         u[0]=u[Wimax-2];
         u[Wimax-1]=u[1];
      }
      
      t+=DT*Dispstep;
      draw();
   }
}


/**********************  $B=i4|>r7o(B *************************/

#define WinX(j) ((0.5-(double)(j)/(double)Wjmax)/0.3)
#define WinJ(x) ((int)(Wjmax*(-0.3*(x)+0.5)))		  
		  
void initial(int n)
{
   static int i0=(int)(0.6*(double)Wimax);
   static double up=1;
   
   if(n){
       i0=W.ev.xbutton.x;
       up=WinX(W.ev.xbutton.y);
       W.chev=0;
   }

   for(int i=0; i<Wimax; i++)      u[i]=-1;   
   if(i0>=Wimax/2){
      for(int i=Wimax-i0; i<i0; i++)  u[i]=up;
   }else{
      for(int i=i0; i<Wimax-i0; i++)  u[i]=up;
   }    
   
   u[0]=u[Wimax-2];
   u[Wimax-1]=u[1];
   t=0; Initial=1;
   draw();
   PW_draw();
}


/********************** $B=q$/(B *************************/

void draw(void)
{
    /*======= snapshot =========*/
    int j,j0;
    if(Initial){
	W.clear();
	W.draw_line(2,Wimax/2,0,Wimax/2,Wjmax);	
	W.draw_line(2,0,WinJ(0),Wimax,WinJ(0));
	W.draw_line(1,0,WinJ(1),Wimax,WinJ(1));
	W.draw_line(1,0,WinJ(-1),Wimax,WinJ(-1));
        Initial=0;
    }		 

    if(Clear) W.clear(0);
    j0=WinJ(u[0]);
    for(int i=0; i<Wimax; i++){
       j=WinJ(u[i]);
       W.draw_line(0,i-1,j0,i,j);
       j0=j;
    }   
}

#define PWinX(i)  (((double)(i)/(double)PWsize-0.5)/0.3)
#define PWinJ(y)  ((int)(PWsize*(-0.3*(y)+0.5)))		  

void PW_draw(void)
{
    double x;
    int j,j0;

    PW.clear();	
    PW.draw_line(1,PWsize/2,0,PWsize/2,PWsize);	
    PW.draw_line(1,0,PWinJ(0),PWsize,PWinJ(0));

    x=PWinX(0); j0=PWinJ(x-x*x*x-v);
    for(int i=1; i<PWsize; i++){
       x=PWinX(i); j=PWinJ(x-x*x*x-v);
       PW.draw_line(0,i-1,j0,i,j);
       j0=j;
    }
}    




