/*---------------------------------------------------------------------------
  excitable system. 
  It's behavior for large diffusion constant was showed by Ryo Kobayashi.   

     $B"_(Bt u = f(u) - v + $B"$(B u
     $B"_(Bt v = u + D * $B"$(B v
     f(u)=-u+0.5*(tanh((u-a)/$B&D(B)+tanh(a/$B&D(B))
---------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <math.h>
#include <XRS/Control.h>
#include <XRS/Plane.h>
#include <XRS/Graphic.h>

#define random      (drand48()-0.5)
#define f(u)        (-u+0.5*(tanh((u-a)/delta)+tanh(a/delta)))
#define integer(x)  ((int)(floor(x)+floor(2*((x)-floor(x)))))


/************** $BBg0hJQ?t$H4X?t@k8@(B ********************/

double          a,D,delta,r_start;
double          DT,DX;
double          **u,**v;
double          L;
double          t;
int             Wimax,Wjmax,Dispstep=10;

void            initial(int);
void            boundary(void);

/*---------------- $B0J2<(B XWindow $BMQ(B --------------------*/

Control_Window  CW;
Graphic_Window  W,WS(W,0,0);
Plane_Window    PW;
int             show_u=1,PWsize=100;

void            PW_draw(int);
void            explain(int)
{  if(!fork()){ execlp("xdvi","xdvi","-s","8","README_excitable.dvi",0);}  }
double          scale(double x) { return 0.5*(x+0.5);}
double          linear(double x){ return x;}
void            change_uv(int)
{ show_u=!show_u; if(show_u) W<<"u(activator)"; else W<<"v(inihibitor)";}


unsigned long*  color(int n)
{ //---- $BG;$5$,O"B3E*$KJQ$o$k(B2$B<oN`$N?'$r#1$D$NG[Ns$H$7$FMQ0U$9$k!#(B-----
    unsigned long *c=new unsigned long[n];
    for(int i=0; i<n; i++){
      if( i <  n/2 ) c[i]=color_pixel(0.5,1.0-2*(double)i/(double)n);
      else           c[i]=color_pixel(0.6,2*(double)i/(double)n-1.0);
    }
    return c;
}



/********** main ***********/

int main(int argc,char **argv)
{
   /*--------------- initialize ---------------------*/
   
   if(argc>1) L=atof(argv[1]);      else L=200;
   if(argc>2) DX=atof(argv[2]);     else DX=1;
   if(argc>3) DT=atof(argv[3]);     else DT=0.1;
   if(argc>4) a=atof(argv[4]);      else a=0.1;
   if(argc>5) D=atof(argv[5]);      else D=0.01;
   if(argc>6) delta=atof(argv[6]);  else delta=0.02;
   if(argc>7) r_start=atof(argv[7]); else r_start=10;      
   
   fprintf(stderr,"usage%% excitable [L] [DX] [DT] [a] [D] [delta] [r_start]\n"
          " current systemsize: L=%g\n"
          " current parameter : a=%g D=%g delta=%g r_start=%g\n"
	  " space and time meshsize: DX=%g DT=%g\n"
   	  ,L,a,D,delta,r_start,DX,DT); 

   Wimax=Wjmax=(int)(L/DX);  
   array(u,Wimax+1,Wjmax+1); 
   array(v,Wimax+1,Wjmax+1);

   double   **v2,**u2;
   array(u2,Wimax+1,Wjmax+1); 
   array(v2,Wimax+1,Wjmax+1);

/*----- initialize ----------*/
   
   CW.realtime=0;
       f_dialog d(CW,explain);
       d.resize(2*d.width(),d.height());
       d.string(20);
       d.title(" readme (Click here)");
   CW.realtime=1;

   CW<<t<<"t"<<initial<<"initial"<<PARAMETER
     << a << D << delta << r_start 
     <<"a"<<"D"<<"delta"<<"r_start";
   
   W.geometry(Wimax+1,Wjmax+1,1)<<COLOR(color(15),15)<<scale<<change_uv;
   WS.geometry(150,20,1)<<COLOR(color(15),15)<<linear;
   PW.geometry(PWsize,PWsize,1)<<"u-v";
   
   change_uv(0);
   initialize();
   sample_window(WS);

   int i,j,ts;

/*----- main routine --------*/

   double   du,dv,txx;
   
   initial(1);
   PW_draw(1);
   
   while(1){
      checkevent();
      if(CW.chev) PW_draw(0);      
      txx=DT/(DX*DX);

      for(ts=0; ts<Dispstep;ts++){
        for(i=1;i<Wimax;i++)
          for(j=1;j<Wjmax;j++){
             du=u[i-1][j]+u[i+1][j]+u[i][j-1]+u[i][j+1]-4.0*u[i][j];
             u2[i][j]=u[i][j]+txx*du+DT*(f(u[i][j])-v[i][j]);
          }

        for(i=1;i<Wimax;i++)
          for(j=1;j<Wjmax;j++){
             dv=v[i-1][j]+v[i+1][j]+v[i][j-1]+v[i][j+1]-4.0*v[i][j];
             v2[i][j]=v[i][j]+txx*D*dv+DT*u[i][j];
          }

        for(i=1;i<Wimax;i++)
          for(j=1;j<Wjmax;j++){
             u[i][j]=u2[i][j];
             v[i][j]=v2[i][j];	     
          }
	
        boundary();
      }
      
      t+=DT*Dispstep;
      if(show_u) W.draw_2D(u);
      else       W.draw_2D(v);
   }
}


/**********************  $B=i4|>r7o(B *************************/

void initial(int n)
{
    int ii,jj;

    for(int j=0; j<Wjmax+1; j++)
        for(int i=0; i<Wimax+1; i++){ u[i][j] = 0; v[i][j] = 0; }

    switch(n){
    case 1:
       for(int j=1; j<Wjmax; j++)
         for(int i=1; i<Wimax; i++){
	    ii=i-Wimax/2; jj=j-Wjmax/2;
            if(ii*ii+jj*jj<(r_start/DX*r_start/DX)) v[i][j] = -1.0;
         }
       break;       
    case 2:
       for(int j=1; j<Wjmax; j++)
         for(int i=1; i<Wimax; i++){
	    ii=i-Wimax/2; jj=j-Wjmax/2;
            if(abs(jj)<r_start/DX && abs(ii)<Wimax/4) v[i][j] = -1.0;
         }
       break;
    default:
       for(int j=1; j<Wjmax; j++)
         for(int i=1; i<Wimax; i++){
	    ii=i-Wimax/2; jj=j-Wjmax/2;
            if(abs(jj)<r_start/DX && abs(ii)>Wimax/4) v[i][j] = -1.0; 
         }
       break;
    }
       
    boundary();
    t=0;
}


/**********************  $B6-3&>r7o(B *************************/

void boundary(void)
{
  /*---- periodic boundary ----*/
    for(int j=1;j<Wjmax;j++){
        v[0][j]=v[Wimax-1][j]; u[0][j]=u[Wimax-1][j];
        v[Wimax][j]=v[1][j];   u[Wimax][j]=u[1][j];	
    }
    
    for(int i=1;i<Wimax;i++){
        v[i][0]=v[i][Wjmax-1]; u[i][0]=u[i][Wjmax-1];
        v[i][Wjmax]=v[i][1];   u[i][Wjmax]=u[i][1];
    }
}


#define PWinX(i)  (((double)(i)/(double)PWsize-0.5)/0.3)
#define PWinI(x)  ((int)(PWsize*(-0.3*(x)+0.5)))		  

void PW_draw(int)
{
    double x;
    int j0,j;

    PW.clear();	
    PW.draw_line(1,PWsize/2,0,PWsize/2,PWsize);	
    PW.draw_line(1,0,PWinI(0),PWsize,PWinI(0));

    x=PWinX(0); j0=PWinI(f(x));
    for(int i=1; i<PWsize; i++){
        x=PWinX(i); j=PWinI(f(x));
	PW.draw_line(0,i-1,j0,i,j);
	j0=j;
      }
}    








    


